# OpenFairViewer
OpenFairViewer - a FAIR, ISO and OGC (meta)data compliant GIS data viewer for browsing, accessing and sharing geo-referenced statistics
[![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.2249305.svg)](https://doi.org/10.5281/zenodo.2249305)

## Install OpenFairViewer

The OpenFairViewer is a HTML5/JS map viewer application that operates standard browsing, access, query and sharing of geospatial data. 

To deploy and configure an OpenFairViewer, you should install [node.js](https://nodejs.org/en/download/) that will provide the ``npm`` command required to build your OpenFairViewer project. Once node.js installed, install the following packages:

```
npm instal parcel-bundler --save-dev
npm install parcel-plugin-static-files-copy  --save-dev
```

Next, create a project with ``npm init``, or create the below ``package.json`` as follows:

```json
{
  "name": "openfairviewer_test",
  "version": "1.0.0",
  "description": "My mapviewer powered by OpenFairViewer",
  "main": "main.js",
  "scripts": {
    "test": "echo \"Error: no test specified\" && exit 1",
    "start": "parcel index.html",
    "build": "parcel build --public-url ./ index.html"
  },
  "staticFiles": {
    "staticPath": [
      {
        "staticPath": "node_modules/openfairviewer/src/templates",
        "staticOutDir": "templates"
      },
      {
        "staticPath": "node_modules/openfairviewer/src/img",
        "staticOutDir": "img"
      }
    ]
  },
  "author": "Your name",
  "license": "licence",
  "devDependencies": {
    "cssnano": "^4.1.10",
    "parcel-bundler": "^1.12.4",
    "parcel-plugin-static-files-copy": "^2.5.0"
  },
  "dependencies": {

  }
}

```

Next, install OpenFairViewer using:

```
npm install eblondel/openfairviewer#2.0.2
```

After the installation, OpenFairViewer will be installed as dependency.

## Configure OpenFairViewer

The initialization of the viewer is done with a Javascript only, with a single ``main.js``. A simple ``index.html`` file is required referencing this javascript file, then all the HTML markup required is generated by OpenFairViewer. As OpenFairViewer is _metadata-driven_, the minimal configuration is done by specifying the OGC Catalogue Service for the Web (CSW) endpoint URL.

```javascript
import $ from 'jquery';
window.$ = $;

import OpenFairViewer from 'openfairviewer/src/OpenFairViewer.js';
 
window.OFV = null;
$(document).ready(function(){
	new OpenFairViewer({
		id: 'OFV',
		profile : '<p>Welcome to my <b>data portal!</b> powered by OpenFairViewer</p>',
		OGC_CSW_BASEURL: "https://localhost:8080/geonetwork/srv/eng/csw"
	}).init(true);
	console.log(OFV);
});

```

Next, you can run ``npm start``, it will compile deploy your viewer to http://localhost:1234

To create a build for deployment on web servers, run ``npm run build``. The compiled web resources will be created in the ``dist`` directory, and can be copied to your webserver.


## Options

The options of OpenFairViewer are grouped according to the main component of application, ie **browse**, **query**, **map**

An OpenFairViewer instance can be then customized handling one more options like this:

```javascript
import $ from 'jquery';
window.$ = $;

import OpenFairViewer from 'openfairviewer/src/OpenFairViewer.js';
 
window.OFV = null;
$(document).ready(function(){
	new OpenFairViewer({
		//an identifier used for the OpenFairViewer instance
		id: 'OFV', 
		//a profile (plain html markup or text) used as About window opened on viewer start
		profile : '<p>Welcome to my <b>data portal!</b> powered by OpenFairViewer</p>',
		//an OGC CSW endpoint URL
		OGC_CSW_BASEURL: "https://localhost:8080/geonetwork/srv/eng/csw"
	},{
		labels: { <label options here> },
		find: { <browse options here> },
		access: { <browse options here> },
		map: { <map options here> }
	}).init(true);
	console.log(OFV);
});

```

The next sections highlight the current options available in OpenFairViewer

### _labels_ options

Application vocabulary used for customization of labels or forthcoming support of i18n. If ignored, default (English) labels will be applied

### _find_ options

Name | Type | Description| Default value
-----|------|------------|----------------
filters | ``Array``| An array of filters applied with 'AND' condition. Filters are defined are defined according to CSW capabilities. Each filter can be defined as object ```{name: <property>, value: <value>}```, eg. all datasets whose identifier contains the string 'species' ```{name: "dc:identifier", value: "%species%" }``` | _empty_
filterByWMS| ``Boolean``| Indicates if only datasets with OGC WMS map resources should be listed | false
datasetInfoHandler| ``function`` | A function taking as parameter a metadata object. By default it will open a new tab with | Function returning CSW GetRecordByID XML in new tab
maxitems| ``Integer``| Maximum number of datasets to retrieve from the catalogue| null


### _access_ options

Name | Type | Description| Default value
-----|------|------------|----------------
columns | ``Integer``| Number of columns to use for displaying the dynamic query form. **_Experimental_**| 1
time | ``String``| Name of the widget to use for time selection. Default is 'datePicker'. Possible alternative value 'slider' for time slider. **_Experimental_** | 'datePicker'
dashboard | ``boolean`` | Sub options for dasbhoard. ``enabled``: Activate the capacity to plug dashboards
dashboard | ``function`` | Sub options for dasbhoard. ``handler``: A function with arg ``layer`` to return dashboards as plain html. To use R shiny app dashboards, you can import the ``OpenFairShiny.js`` that provides utilities to load Shiny apps as iFrames. This handler allows to condition the plug of different dashboard handlers depending on the layers. By default, no dashboard will be associated to the layer. See below example. 

Example of dashboard handler:

```javascript
function(layer){
	if(layer.id.startsWith('my_layer')){
		//for my_layer we want to have a R shiny dashboard access
		return OpenFairShiny.dashboardHandler("https://myshinyproxy.org/app_direct/myShinyApp/", layer, false);
	}else{
		//for other layers, no dashboard
		return;
	}
}
```

### _map_ options

Name | Type | Description| Default value
-----|------|------------|----------------
extent | ``Array``| Bounding box to use to bind the map geographic extent | [-180, -90, 180, 90]
zoom | ``Integer``| Zoom level to use for the map | 3
projection|``String``|Default map projection|'EPSG:4326'
proj4defs|``Array``|An array of Proj4 projection definitions in the form ``{epsgcode: "here the epsgcode", proj4string, : "here the proj4 string definition"}``|empty
layergroups|``Array``| Array of layer groups. Each layer group should be an object in the form  ``{name: "<layergroup name>", fold: 'open'}`` where fold is open/close depending if the layer group should be open or close at start| [{name: "Base overlays", fold: 'close'},{name: "Statistical maps", fold: 'open'}]
mainlayergroup|``Integer``|Index within ``layergroups`` to define the main layer group (where layers loaded from the ``browse`` part will be added) | 1
baselayers |``Array``| An array of baselayers defined with OpenLayers 6 layer objects| [UN Geospatial](https://geoservices.un.org) baselayers
overlays|``Array``| Array of base overlays to add to the map. Each overlay is defined as simplified object ``{group: <mainOverlayGroup>, id: <id>,title: <title>, wmsUrl: <wmsUrl>, wmsVersion: <wmsVersion>, layer: <layer>, hidden: <true/false>, visible: <true/false>, showLegend: <true/false>, opacity: <0 to 1>, tiled: <true/false>, cql_filter: <cql_filter>, style: <style>}``. | _empty_
aggregated_layer_suffix [DEPRECATED]|``String``| Layer name suffix used to characterize aggregated layers| "_aggregated"
attribution|``String``|A string giving html markup for the map attribution.|null
styling|``Object``| Sub options for dynamic styling in geo-referenced statistics. ``dynamic``: true/false, ``breaks`` for labeling class interval breaks [""," to ",""]|
tooltip|``Object``| Sub options for map layer tooltip/popup. ``enabled``: true/false, ``handler``: function with parameters ``(layer, feature)`` for handling tooltip content. Default will list attributes of features. The tooltip is operated through WMS GetFeatureInfo protocol, hence requires that the data protocol uses the same protocol as the viewer (no mix of http/https origin) and is CORS enabled to allow GetFeatureInfo XMLHttpRequest to be performed|
tooltip|``Object``| Sub options for map layer tooltip/popup. ``handler``: A function with args ``layer``/ ``feature`` to return popup content as plain html. To use R shiny app as popups, you can import the ``OpenFairShiny.js`` that provides utilities to load Shiny apps as iFrames. This handler allows to condition the plug of different popup handlers depending on the layers. By default, a simple handler (DEFAULT_HANDLER) will be provided by OpenFairViewer. See below example.


Example of popup handler:

```javascript
function(layer, feature){
	if(layer.id.startsWith('my_layer')){
			return OpenFairShiny.popupHandler("https://myshinyproxy.org/app_direct/myShinyPopup/", layer, feature, false);
		}else{
			return this.DEFAULT_HANDLER(layer, feature);
		}
	}
}
```
